@@ -34,7 +34,7 @@ pub enum Error {
     #[error("At least one input is missing an ownership proof")]
     MissingInputOwnerProof,
     #[error("Mint request doesn't balance out sum(input) == sum(output)")]
-    DbcReissueRequestDoesNotBalance { input: u64, output: u64 },
+    DbcReissueRequestDoesNotBalance,
     #[error("Outputs must be numbered 0..N where N = # of outputs")]
     OutputsAreNotNumberedCorrectly,
     #[error("Failed to unblind an input DBC")]
@@ -55,12 +55,31 @@ pub enum Error {
     DbcContentNotPresentInTransactionOutput,
     #[error("Dbc Content parents is not the same transaction inputs")]
     DbcContentParentsDifferentFromTransactionInputs,
+
+    #[error("RangeProof error: {0}")]
+    RangeProof(#[from] bulletproofs::ProofError),
+
+    #[error("Decryption error: {0}")]
+    DecryptionBySharesFailed(#[from] blsttc::error::Error),
+
+    #[error("Decryption failed")]
+    DecryptionBySecretKeyFailed,
+
+    #[error("Invalid AmountSecret bytes")]
+    AmountSecretsBytesInvalid,
+
+    #[error("Invalid Amount Commitment")]
+    AmountCommitmentInvalid,
+
     /// I/O error.
     #[error("I/O error: {0}")]
     Io(#[from] io::Error),
     /// JSON serialisation error.
-    #[error("JSON serialisation error:: {0}")]
+    #[error("JSON serialisation error: {0}")]
     JsonSerialisation(#[from] serde_json::Error),
     #[error("SpendBook error {0}")]
     SpendBook(String),
+
+    #[error("Infallible.  Can never fail")]
+    Infallible(#[from] std::convert::Infallible),
 }
